import Foundation

var fileContents = (try? String(contentsOfFile: "todo.txt", encoding: NSUTF8StringEncoding)) ?? ""
var items = fileContents.componentsSeparatedByString("\n")
items = items.filter { $0 != "" }

func help() {
	print("Available commands:")
	print("   add \"Buy new socks\"")
	print("   delete 3")
	print("   list")
	print("   search \"socks\"")
	print("")
}

func add(item: String) {
	items.append(item)
	save()
}

func delete(number: Int) {
	guard number > 0 && number <= items.count else {
		print("That item doesn't exist!")
		return
	}

	items.removeAtIndex(number - 1)
	save()
}

func search(item: String) {
	let splitSearch = item.lowercaseString.componentsSeparatedByString(" ")
	var matchingItems = [String]()

	for item in items {
		var itemMatchesSearch = true
		let lowerItem = item.lowercaseString

		for word in splitSearch {
			if !lowerItem.containsString(word) {
				itemMatchesSearch = false
				break
			}
		}

		if itemMatchesSearch {
			matchingItems.append(item)
		}
	}

	list(matchingItems)
}

func save() {
	let joined = items.joinWithSeparator("\n")

	do {
		try joined.writeToFile("todo.txt", atomically: true, encoding: NSUTF8StringEncoding)
	} catch {
		print("Failed to write todo.txt - please check permissions.")
	}
}

func list(listItems: [String]) {
	guard listItems.count > 0 else {
		print("Your todo list is empty!")
		return
	}

	for (index, item) in listItems.enumerate() {
		print("\(index + 1). \(item)")
	}
}

if Process.arguments.count == 1 {
	help()
} else {
	switch Process.arguments[1] {
	case "add":
		if Process.arguments.count == 3 {
			add(Process.arguments[2])
		} else {
			help()
		}
	case "delete":
		if Process.arguments.count == 3, let number = Int(Process.arguments[2]) {
			delete(number)
		} else {
			help()
		}
	case "list":
		list(items)
	case "search":
		if Process.arguments.count == 3 {
			search(Process.arguments[2])
		} else {
			help()
		}
	default:
		help()
	}
}